package pqrtree;

/**
 * A <code>ChildrenDoublyLinkedList</code>
 * is an object very similar to a <code>DoublyLinkedList</code>,
 * except by the kind of data its nodes stores, just
 * PQR tree nodes.
 * 
 * @version 0.1
 * @author Bruno Dilly
 * @see pqrtree.DoublyLinkedList
 */
public class ChildrenDoublyLinkedList extends DoublyLinkedList {

	/**
	 * Instanciates a new children list.
	 * It's constructed just like <code>DoublyLinkedList</code>
	 * @see DoublyLinkedList#DoublyLinkedList()
	 */
	public ChildrenDoublyLinkedList ()
	{
		super();
	}
	
	/**
	 * Verifies if all the nodes painted with a determined color
	 * is consecutive.
	 * @param color The color of the node.
	 * @return <code>true</code>, if all nodes colored with
	 * a determined color are consecutive, or <code>false</code>, 
	 * if they aren't.
	 */
	public boolean areAllColoredChildrenConsecutive(int color)
	{
		int s = 0;
		ListNode aux = head;
		ListNode iterator = startIteration();
		while (aux!=null)
		{
			int nodeColor = ((Node)aux.getData()).getColor();
			if ((nodeColor != color & s == 1) | (nodeColor == color & s == 0))
			{
				s++;
			}
			else if ( nodeColor == color & s == 2)
			{
				return false;
			}
			aux=aux.getNext(iterator);
		}
		return true;
	}
}
