package pqrtree;

/**
 * A <code>ListNode</code> is the element of the
 * <code>DoublyLinkedList</code>, consisting of
 * a pointer to its buddy, to the next node,
 * and to another object.
 * 
 * @version 0.1
 * @author Bruno Dilly
 * @see pqrtree.DoublyLinkedList
 */
public class ListNode {
	private ListNode dir1;
	private ListNode dir2;
	private Object data;
	
	/**
	 * Creates a list node that stores
	 * an object.
	 * @param data Object that must be stored.
	 */
	public ListNode(Object data)
	{
		dir1 = null;
		dir2 = null;
		this.data = data;
	}
	
	/**
	 * Gets the next element of the list node.
	 * @param iterator The iterator is a list node that stores
	 * the previous list node, or the head, when its the first
	 * step of a loop.
	 * @return Next list node.
	 */
	public ListNode getNext(ListNode iterator) 
	{
		if (iterator.getData() == this) // this is the begining if iteration
		{
			if (dir1 != null)
			{
				return dir1;
			}
			else
			{
				return dir2;
			}
		}
		else if (dir1 != iterator.getData())
		{
			iterator.setData(this);
			return dir1;
		}
		else
		{
			iterator.setData(this);
			return dir2;
		}
	}

	/**
	 * Sets the neighbor element of the list node in direction 1.
	 * @param n Neighbor list node.
	 */
	public void setDir1(ListNode n)	
	{
		dir1 = n;
	}
	
	/**
	 * Sets the neighbor element of the list node in direction 2.
	 * @param n Neighbor list node.
	 */
	public void setDir2(ListNode n)	
	{
		dir2 = n;
	}
	
	/**
	 * Getss the neighbor element of the list node in direction 1.
	 * @return Neighbor list node.
	 */
	public ListNode getDir1()
	{
		return dir1;
	}
	
	/**
	 * Gets the neighbor element of the list node in direction 2.
	 * @return Neighbor list node.
	 */
	public ListNode getDir2()
	{
		return dir2;
	}
	
	/**
	 * Gets the data stored by the list node,
	 * usually a tree node, or another list node.
	 * @return The object stored, a list or tree node.
	 */
	public Object getData() 
	{
		return data;
	}
	
	public void setData(Object o)
	{
		data = o;
	}
	
	/**
	 * Remove the node of the list it belongs.
	 */
	public void removeMe()
	{
		if (dir1 != null)
		{
			if (dir1.getDir1() == this)
			{
				dir1.setDir1(dir2);
			}
			else // dir2.getDir2() == this
			{
				dir1.setDir2(dir2);
			}
		}
		if (dir2 != null)
		{
			if (dir2.getDir1() == this)
			{
				dir2.setDir1(dir1);
			}
			else // dir2.getDir2() == this
			{
				dir2.setDir2(dir1);
			}
		}
	}	
}
