package pqrtreeapplet;

import java.awt.*;
import javax.swing.*;
import pqrtree.PQRTree;

/**
 * The secondary view component of the model/view/controller for this application
 * This class is the drawing area for the PQRTree.
 * 
 * Based in the PQPanel class of Jon Harris' Graphical Implementation of 
 * PQ Tree Data Structure.
 * <a href="http://www.jharris.ca/portfolio/accode/PQTree.html">PQ Tree</a>
 * link.
 * 
 * @version 0.1
 * @author Bruno Dilly
 * 
 */
public class PQRPanel extends JPanel
{
	PQRTree pqrTree;
	static final long serialVersionUID = 2409;
	
	/**
	 * Creates new panel.
	 * @param pqrTree
	 * @throws Exception
	 */
	public PQRPanel(PQRTree pqrTree) throws Exception
	{
		super();
		this.pqrTree = pqrTree;
		setBackground(Color.white);
		refreshTree(pqrTree, false);
	}
	
	/**
	 * Drawns the tree.
	 * @param aPen The graphics object pen.
	 */
	public void paint(Graphics aPen)
	{
		try
		{
			super.paint(aPen);
			if (pqrTree != null)
			{
				pqrTree.drawTree(aPen);
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * 
	 * @param pqrTree The tree created by the controller.
	 * @param now If <code>true</code>, paints the tree immediately, 
	 * or in other case, repaint.
	 * @throws Exception
	 */
	public void refreshTree(PQRTree pqrTree, boolean now) throws Exception
	{
		this.pqrTree = pqrTree;
		pqrTree.prepareToDrawTree();
		setSize(pqrTree.getWidth(), pqrTree.getHeight());
		setPreferredSize(new Dimension(pqrTree.getWidth(), pqrTree.getHeight()));
		revalidate();
		if ( now )
		{
			paintImmediately(0,0,pqrTree.getWidth(),pqrTree.getHeight());
		}
		else
		{
			repaint();
		}
	}
}
