package pqrtreeapplet;

import java.awt.*;
import javax.swing.*;
import pqrtree.PQRTree;

/**
 * The primary view component of the model/view/controller for this application.
 * This class represents the GUI for performing and displaying reductions on
 * a PQRTree.
 * 
 * Based in the PQWindow class of Jon Harris' Graphical Implementation of 
 * PQ Tree Data Structure.
 * <a href="http://www.jharris.ca/portfolio/accode/PQTree.html">PQ Tree</a>
 * link.
 * 
 * @version 0.1
 * @author Bruno Dilly
 *
 */
public class PQRWindow extends JPanel
{
	
	static final long serialVersionUID = 2509;
	private PQRPanel pqrPanel;
	private JList leafList;
	private JButton reduceButton;
	private JButton stepButton;
	private JButton resetButton;
	private JTextField pauseField;
	private JCheckBox pauseBox;
	private JTextArea outputArea;
	private JSplitPane vSplitPane;
	
	public JButton getReduceButton() { return reduceButton; }
	public JButton getStepButton() { return stepButton; }
	public JList getReduceList() { return leafList; }
	public JButton getResetButton() { return resetButton; }
	public JTextField getPauseField() { return pauseField; }
	public JCheckBox getPauseBox() { return pauseBox; }
	public JTextArea getOutputArea() { return outputArea; }
	
	/**
	 * Creates the window to displays the tree.
	 * @param pqrTree The tree to be displayed.
	 * @param drawAllSteps 
	 * @param delayInSeconds
	 * @throws Exception
	 */
	public PQRWindow(PQRTree pqrTree, boolean drawAllSteps, int delayInSeconds) throws Exception
	{
		pqrPanel = new PQRPanel(pqrTree);
		leafList = new JList(pqrTree.getLeaves());
		reduceButton = new JButton("Reduce");
		stepButton = new JButton("Step");
		resetButton = new JButton("Reset");
		pauseField = new JTextField();
		pauseField.setText(String.valueOf(delayInSeconds));
		pauseBox = new JCheckBox("Show All Steps");
		pauseBox.setSelected(drawAllSteps);
		outputArea = new JTextArea();
		
		GridBagLayout layout = new GridBagLayout(); 
		GridBagConstraints layoutCons = new GridBagConstraints(); 
		setLayout(layout);
		
		GridBagLayout topLayout = new GridBagLayout(); 
		GridBagConstraints topLayoutCons = new GridBagConstraints(); 
		
		JPanel topPanel = new JPanel();
		topPanel.setLayout(topLayout);
		
		JScrollPane scrollPane = new JScrollPane(pqrPanel,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		topLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		topLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		topLayoutCons.gridwidth = GridBagConstraints.REMAINDER;
		topLayoutCons.gridheight = 300; 
		topLayoutCons.fill = GridBagConstraints.BOTH; 
		topLayoutCons.insets = new Insets(3, 3, 3, 3); 
		topLayoutCons.anchor = GridBagConstraints.NORTH; 
		topLayoutCons.weightx = 1.0; 
		topLayoutCons.weighty = 1.0; 
		topLayout.setConstraints(scrollPane, topLayoutCons);
		topPanel.add(scrollPane);
		
		GridBagLayout bottomLayout = new GridBagLayout(); 
		GridBagConstraints bottomLayoutCons = new GridBagConstraints(); 
		
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(bottomLayout);
		
		
		JScrollPane scrollPane2 = new JScrollPane(leafList,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		bottomLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		bottomLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		bottomLayoutCons.gridwidth = 20;
		bottomLayoutCons.gridheight = GridBagConstraints.REMAINDER; 
		bottomLayoutCons.fill = GridBagConstraints.BOTH; 
		bottomLayoutCons.insets = new Insets(3, 3, 3, 3); 
		bottomLayoutCons.anchor = GridBagConstraints.NORTH; 
		bottomLayoutCons.weightx = 1.0; 
		bottomLayoutCons.weighty = 1.0; 
		bottomLayout.setConstraints(scrollPane2, bottomLayoutCons);
		bottomPanel.add(scrollPane2);
		
		GridBagLayout controlLayout = new GridBagLayout(); 
		GridBagConstraints controlLayoutCons = new GridBagConstraints(); 
		
		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(controlLayout);
		
		controlLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridwidth = 10;
		controlLayoutCons.gridheight = 10; 
		controlLayoutCons.fill = GridBagConstraints.HORIZONTAL; 
		controlLayoutCons.insets = new Insets(3, 3, 3, 3); 
		controlLayoutCons.anchor = GridBagConstraints.NORTH; 
		controlLayoutCons.weightx = 1.0; 
		controlLayoutCons.weighty = 0.0; 
		controlLayout.setConstraints(reduceButton, controlLayoutCons);
		controlPanel.add(reduceButton);
		
		controlLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridwidth = GridBagConstraints.REMAINDER;
		controlLayoutCons.gridheight = 10; 
		controlLayoutCons.fill = GridBagConstraints.HORIZONTAL; 
		controlLayoutCons.insets = new Insets(3, 3, 3, 3); 
		controlLayoutCons.anchor = GridBagConstraints.NORTH; 
		controlLayoutCons.weightx = 1.0; 
		controlLayoutCons.weighty = 0.0; 
		controlLayout.setConstraints(stepButton, controlLayoutCons);
		controlPanel.add(stepButton);
		
		GridBagLayout pauseLayout = new GridBagLayout(); 
		GridBagConstraints pauseLayoutCons = new GridBagConstraints(); 
		
		JPanel pausePanel = new JPanel();
		pausePanel.setLayout(pauseLayout);
		
		JLabel pauseLabel = new JLabel("Pause in Seconds:");
		
		pauseLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		pauseLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		pauseLayoutCons.gridwidth = 5;
		pauseLayoutCons.gridheight = 10; 
		pauseLayoutCons.fill = GridBagConstraints.BOTH; 
		pauseLayoutCons.insets = new Insets(3, 3, 3, 3); 
		pauseLayoutCons.anchor = GridBagConstraints.NORTH; 
		pauseLayoutCons.weightx = 0.5; 
		pauseLayoutCons.weighty = 0.25; 
		pauseLayout.setConstraints(pauseLabel, pauseLayoutCons);
		pausePanel.add(pauseLabel);
		
		pauseLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		pauseLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		pauseLayoutCons.gridwidth = 10;
		pauseLayoutCons.gridheight = 10; 
		pauseLayoutCons.fill = GridBagConstraints.BOTH; 
		pauseLayoutCons.insets = new Insets(3, 3, 3, 3); 
		pauseLayoutCons.anchor = GridBagConstraints.NORTH; 
		pauseLayoutCons.weightx = 1.0; 
		pauseLayoutCons.weighty = 0.25; 
		pauseLayout.setConstraints(pauseField, pauseLayoutCons);
		pausePanel.add(pauseField);
		
		pauseLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		pauseLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		pauseLayoutCons.gridwidth = GridBagConstraints.REMAINDER;
		pauseLayoutCons.gridheight = 10; 
		pauseLayoutCons.fill = GridBagConstraints.BOTH; 
		pauseLayoutCons.insets = new Insets(3, 3, 3, 3); 
		pauseLayoutCons.anchor = GridBagConstraints.NORTH; 
		pauseLayoutCons.weightx = 0.5; 
		pauseLayoutCons.weighty = 0.25; 
		pauseLayout.setConstraints(pauseBox, pauseLayoutCons);
		pausePanel.add(pauseBox);
		
		controlLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridwidth = 10;
		controlLayoutCons.gridheight = 10; 
		controlLayoutCons.fill = GridBagConstraints.HORIZONTAL; 
		controlLayoutCons.insets = new Insets(3, 3, 3, 3); 
		controlLayoutCons.anchor = GridBagConstraints.NORTH; 
		controlLayoutCons.weightx = 1.0; 
		controlLayoutCons.weighty = 0.0; 
		controlLayout.setConstraints(pausePanel, controlLayoutCons);
		controlPanel.add(pausePanel);
		
		controlLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		controlLayoutCons.gridwidth = GridBagConstraints.REMAINDER;
		controlLayoutCons.gridheight = 10; 
		controlLayoutCons.fill = GridBagConstraints.HORIZONTAL; 
		controlLayoutCons.insets = new Insets(3, 3, 3, 3); 
		controlLayoutCons.anchor = GridBagConstraints.NORTH; 
		controlLayoutCons.weightx = 1.0; 
		controlLayoutCons.weighty = 0.0; 
		controlLayout.setConstraints(resetButton, controlLayoutCons);
		controlPanel.add(resetButton);

		bottomLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		bottomLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		bottomLayoutCons.gridwidth = 20;
		bottomLayoutCons.gridheight = 20; 
		bottomLayoutCons.fill = GridBagConstraints.BOTH; 
		bottomLayoutCons.insets = new Insets(3, 3, 3, 3); 
		bottomLayoutCons.anchor = GridBagConstraints.NORTH; 
		bottomLayoutCons.weightx = 1.0; 
		bottomLayoutCons.weighty = 0.0; 
		bottomLayout.setConstraints(controlPanel, bottomLayoutCons);
		bottomPanel.add(controlPanel);
		
		JScrollPane scrollPane3 = new JScrollPane(outputArea,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		bottomLayoutCons.gridx = GridBagConstraints.RELATIVE; 
		bottomLayoutCons.gridy = GridBagConstraints.RELATIVE; 
		bottomLayoutCons.gridwidth = GridBagConstraints.REMAINDER;
		bottomLayoutCons.gridheight = GridBagConstraints.REMAINDER; 
		bottomLayoutCons.fill = GridBagConstraints.BOTH; 
		bottomLayoutCons.insets = new Insets(3, 3, 3, 3); 
		bottomLayoutCons.anchor = GridBagConstraints.NORTH; 
		bottomLayoutCons.weightx = 1.0; 
		bottomLayoutCons.weighty = 0.5; 
		bottomLayout.setConstraints(scrollPane3, bottomLayoutCons);
		bottomPanel.add(scrollPane3);
		
		vSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				topPanel, bottomPanel);
		vSplitPane.setOneTouchExpandable(true);
		
		layoutCons.gridx = GridBagConstraints.RELATIVE; 
		layoutCons.gridy = GridBagConstraints.RELATIVE; 
		layoutCons.gridwidth = GridBagConstraints.REMAINDER;
		layoutCons.gridheight = GridBagConstraints.REMAINDER; 
		layoutCons.fill = GridBagConstraints.BOTH; 
		layoutCons.insets = new Insets(0, 0, 0, 0); 
		layoutCons.anchor = GridBagConstraints.NORTH; 
		layoutCons.weightx = 1.0; 
		layoutCons.weighty = 1.0; 
		layout.setConstraints(vSplitPane, layoutCons);
		add(vSplitPane);
		refreshTree(pqrTree, false);
	}
	
	/**
	 * Refreshs the tree in the panel.
	 * @param pqrTree The tree to be refreshed.
	 * @param now If <code>true</code>, paints the tree immediately, 
	 * or in other case, repaint. To be passed to the panel.
	 * @throws Exception
	 * @see PQRPanel#refreshTree(PQRTree, boolean)
	 */
	public void refreshTree(PQRTree pqrTree, boolean now) throws Exception
	{
		pqrPanel.refreshTree(pqrTree, now);
		pqrPanel.revalidate();
	}
	
	/**
	 * Refreshs the list with the leaves of the tree as elements.
	 * @param pqrTree The tree that has the elements.
	 */
	public void refreshList(PQRTree pqrTree)
	{
		leafList.setListData(pqrTree.getLeaves());
	}
	
	/**
	 * Sets the divider location.
	 * @param location The new position.
	 */
	public void setDividerLocation(int location)
	{
		vSplitPane.setDividerLocation(location);
	}
	
	/**
	 * Sets the divider location.
	 * @param percent The percent representing the position.
	 */
	public void setDividerLocation(double percent)
	{
		vSplitPane.setDividerLocation(percent);
	}
}
